@@ -1,6 +1,7 @@
 from pathlib import Path
+import warnings
 import glob
-from typing import Union, Optional
+from typing import Union, Optional, List
 
 import pandas as pd
 from fmu.ensemble import ScratchEnsemble, EnsembleSet
@@ -128,7 +129,11 @@ def find_sens_type(senscase: str) -> Optional[str]:
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
 @webvizstore
 def find_surfaces(
-    ensemble_paths: dict, suffix: str = "*.gri", delimiter: str = "--"
+    ensemble_paths: dict,
+    surface_folder: str = "share/results/maps",
+    surface_files: Optional[List] = None,
+    suffix: str = "*.gri",
+    delimiter: str = "--",
 ) -> pd.DataFrame:
     """Reads surface file names stored in standard FMU format, and returns a dictionary
     on the following format:
@@ -142,19 +147,35 @@ def find_surfaces(
     """
     # Create list of all files in all realizations in all ensembles
     files = []
-    for path in ensemble_paths.values():
-        path = Path(path)
-        for realpath in glob.glob(str(path / "share" / "results" / "maps" / suffix)):
-            stem = Path(realpath).stem.split(delimiter)
-            if len(stem) >= 2:
-                files.append(
-                    {
-                        "path": realpath,
-                        "name": stem[0],
-                        "attribute": stem[1],
-                        "date": stem[2] if len(stem) >= 3 else None,
-                    }
-                )
+    for _, ensdf in get_realizations(ensemble_paths=ensemble_paths).groupby("ENSEMBLE"):
+        ens_files = []
+        for _real_no, realdf in ensdf.groupby("REAL"):
+            runpath = realdf.iloc[0]["RUNPATH"]
+            for realpath in glob.glob(str(Path(runpath) / surface_folder / suffix)):
+                filename = Path(realpath)
+                if surface_files and filename.name not in surface_files:
+                    continue
+                stem = filename.stem.split(delimiter)
+                if len(stem) >= 2:
+                    ens_files.append(
+                        {
+                            "path": realpath,
+                            "name": stem[0],
+                            "attribute": stem[1],
+                            "date": stem[2] if len(stem) >= 3 else None,
+                            **realdf.iloc[0],
+                        }
+                    )
+        if not ens_files:
+            warnings.warn(f"No surfaces found for ensemble located at {runpath}.")
+        else:
+            files.extend(ens_files)
 
     # Store surface name, attribute and date as Pandas dataframe
+    if not files:
+        raise ValueError(
+            "No surfaces found! Ensure that surfaces file are stored "
+            "at share/results/maps in each ensemble and is following "
+            "the FMU naming standard (name--attribute[--date].gri)"
+        )
     return pd.DataFrame(files)
