@@ -524,8 +524,8 @@ def make_heatmap(
     uirevision: str = None,
     showscale: bool = False,
     reverse_y: bool = True,
-    xaxis: np.array = None,
-    yaxis: np.array = None,
+    xaxis: np.ndarray = None,
+    yaxis: np.ndarray = None,
     text: str = None,
     title: str = None,
     yaxis_title: str = None,
