@@ -29,13 +29,11 @@ class WellCompletions(WebvizPluginABC):
 
     * **`ensembles`:** Which ensembles in `shared_settings` to visualize.
     * **`compdat_file`:** csvfile with compdat data per realization
-    * **`zone_layer_mapping_file`:** Optional lyr file specifying the zone->layer mapping \
-    * **`well_attributes_file`:** Optional json file with categorical well attributes \
-    * **`kh_unit`:** Optional (default empty)
-    * **`kh_decimal_places`: Optional (default 2)
+    * **`zone_layer_mapping_file`:** Lyr file specifying the zone->layer mapping \
+    * **`well_attributes_file`:** Json file with categorical well attributes \
+    * **`kh_unit`:** Will normally be mDm
+    * **`kh_decimal_places`:**
 
-    The zone layer mapping and well attribute files are intended to be exported from RMS and \
-    there will therefore be one file per realization.
     ---
     The minimum requirement is to define `ensembles`.
 
@@ -49,18 +47,17 @@ class WellCompletions(WebvizPluginABC):
     installed).
     [Link to ecl2csv compdat documentation.](https://equinor.github.io/ecl2df/usage/compdat.html)
 
-    ** Zone layer mapping **
+    **Zone layer mapping**
 
     `zone_layer_mapping_file` file can be dumped to disk per realization by an internal \
     RMS script as part of the FMU workflow. A sample python script will be made available.
 
-    The file needs to be on the lyr format used in Resinsight.
-    [Link to description of lyr format.]\
-    (https://resinsight.org/3d-main-window/formations/#formation-names-description-files-_lyr_)
+    The file needs to be on the lyr format used in ResInsight.
+    [Link to description of lyr format](https://resinsight.org/3d-main-window/formations/#formation-names-description-files-_lyr_).
 
     If no file exists, layers will be used as zones.
 
-    ** Well Attributes file **
+    **Well Attributes file**
 
     `well_attributes_file` file is intended to be generated per realization by an internal \
     RMS script as part of the FMU workflow. A sample script will be made available, but it is \
@@ -68,6 +65,7 @@ class WellCompletions(WebvizPluginABC):
     The categorical well attributes are completely flexible.
 
     The file should be a json file on the following format:
+    ```json
     {
         "version" : "0.1",
         "wells" : [
@@ -95,7 +93,8 @@ class WellCompletions(WebvizPluginABC):
         },
         ]
     }
-    """
+    ```
+    """  # pylint: disable=line-too-long
 
     def __init__(
         self,
@@ -146,9 +145,29 @@ class WellCompletions(WebvizPluginABC):
             ),
         ]
 
+    @property
+    def tour_steps(self) -> list:
+        return [
+            {
+                "id": self.uuid("layout"),
+                "content": "Dashboard vizualizing Eclipse well completion output.",
+            },
+            {"id": self.uuid("ensemble_dropdown"), "content": "Select ensemble."},
+            {
+                "id": self.uuid("well_completions_wrapper"),
+                "content": (
+                    "Visualization of the well completions. "
+                    "Time slider for selecting which time steps to display. "
+                    "Different vizualisation and filtering alternatives are available "
+                    "in the upper right corner."
+                ),
+            },
+        ]
+
     @property
     def layout(self) -> html.Div:
         return html.Div(
+            id=self.uuid("layout"),
             children=[
                 wcc.FlexBox(
                     children=[
@@ -179,7 +198,7 @@ class WellCompletions(WebvizPluginABC):
                 html.Div(
                     id=self.uuid("well_completions_wrapper"),
                 ),
-            ]
+            ],
         )
 
     def set_callbacks(self, app: dash.Dash) -> None:
